{
   (*#####################################################################*
    *
    * Module lexeur pour reprer les logins et les espaces
    *
    * Renvoie les lexmes des logins (pour la cration d'espaces) et les
    * lexmes des espaces (pour le chargement des espaces de discussion
    * au dmarrage du serveur).
    *
    *#####################################################################*)


   open Parseur ;;

   let numero_ligne = ref 1 ;;

   (* permet de renvoyer le lexme n'appartenant pas au langage.
    * On indique aussi le numro de ligne. *)
   exception Lexeme_inconnu of int * string ;;

   (* permet de dire que l'on a rencontr une fin de ligne lors
    * de l'analyse d'un nom d'espace ou de login *)
   exception Fin_ligne_inattendue of int ;;

   let afficher = function s ->
      print_string "lexeur : " ;
      print_endline s ;
      flush stdout 
   ;;
}

(* permet de convertir une chane de caractres en une suite de
 * lexmes logins. *)
rule lexeur = parse
   | '\n'                           { afficher "fin de ligne" ; incr numero_ligne ; lexeur lexbuf }
   | '#'                            { afficher "dbut de commentaire..." ; commentaires lexbuf }

   | "Port"                         { afficher "numro de port" ; Port }
   | "Anonyme"                      { afficher "espace anonyme" ; Anonyme }
   | "Non_anonyme"                  { afficher "espace non-anonyme" ; NonAnonyme }
   | "Prive"                        { afficher "espace priv" ; Prive }
   | "Aucun"                        { afficher "aucun message" ; Aucun }

   | ';'                            { PointVirgule }
   | '['                            { CrochetOuvrant }
   | ']'                            { CrochetFermant }

   | eof                            { afficher "fin du fichier" ; FinFichier }

   | '"'                            { afficher "dbut de chaine..." ; chaine lexbuf }

   | ['0'-'9']+                     { Entier (int_of_string (Lexing.lexeme lexbuf)) }

   | [' ''\t']                      { lexeur lexbuf }

   | _                              { raise (Lexeme_inconnu (!numero_ligne, (Lexing.lexeme lexbuf))) }


and commentaires = parse
   | '\n'                            { afficher "...fin de commentaire (eol)." ; incr numero_ligne ; lexeur lexbuf }
   | eof                             { afficher "fin du fichier" ; FinFichier }
   | _                               { commentaires lexbuf }


and chaine = parse
   | '\n' | '#'                      { raise (Fin_ligne_inattendue !numero_ligne) }
   | [^'"''\n''#' ]*'"'              { let chaine = Lexing.lexeme lexbuf 
                                       in 
                                       afficher ("la chaine est \"" ^ chaine ^ "\"") ;
                                       Chaine (String.sub chaine 0 (String.length chaine - 1)) }
