(* si l'on est en mode debug ou pas *)
let debug = false ;;

(* permet d'ajouter un lment dans une liste trie *)
let rec inserer_dans_liste_triee = 
   function inferieur -> function element -> function liste ->
      let nouvelle_liste = element :: liste
      in
      List.stable_sort inferieur nouvelle_liste
         
(* permet de supprimer un lment de la liste *)
let rec supprimer_element_dans_liste = function element -> function liste ->
   List.find_all (fun e -> e <> element) liste
   
(* permet de trouver la position d'un lment dans une liste *)
let rec position_dans_liste = function element -> function
   | [] -> 
        0 
   | element_courant :: reste ->
        if element_courant = element then
           0
        else
           1 + (position_dans_liste element reste)



class ['a, 'b, 'c] table_connexions 
   creer_objet action_sur_objet 
   est_present 
   nom_de_client nom_de_espace 
   inferieur_sur_clients inferieur_sur_espaces = 

object (self)

   val mutable table = Array.make_matrix 0 0 ((creer_objet ()) : 'c)
   val mutable liste_clients = ([] : 'a list)
   val mutable liste_espaces = ([] : 'b list)


   (* permet de rcuprer la liste des clients *)
   method get_liste_clients = function () -> liste_clients

   (* permet de rcuprer la liste des espaces *)
   method get_liste_espaces = function () -> liste_espaces

   (* permet de rcuprer le tableau de donnes *)
   method get_tableau = function () -> table

   (* dfinit la valeur d'une case de la table *)
   method definir_valeur = function client -> function espace -> function valeur ->
      if List.mem client liste_clients && List.mem espace liste_espaces then
         let position_client = position_dans_liste client liste_clients
         and position_espace = position_dans_liste espace liste_espaces
         in
         table.(position_client).(position_espace) <- 
            action_sur_objet table.(position_client).(position_espace) valeur ;
         begin
            if debug then (self#afficher_table ())
         end
      else
         failwith "table_connexions : definir_valeur, out of range"


   method definir_clients_espace = function espace -> function clients_de_espace ->
      List.iter
         (fun client_courant ->
             self#definir_valeur 
                client_courant espace
                (List.mem client_courant clients_de_espace))
         clients_de_espace

   method definir_tous_espaces = function liste ->
      List.iter
      (fun (espace, clients_espace) ->
          self#definir_clients_espace espace clients_espace)
      liste

   method definir_espaces_client = function client -> function espaces_client ->
      List.iter
      (fun espace_courant ->
          self#definir_valeur 
          client espace_courant
          (List.mem espace_courant espaces_client))
      liste_espaces
      
   method definir_tous_clients = function liste ->
      List.iter
         (fun (client, espaces_client) ->
             self#definir_espaces_client client espaces_client)
         liste


   (* ajout et suppression d'un client ********************************)

   method private ajouter_client_dans_table = function client ->
      let position_client = position_dans_liste client liste_clients
      and nouveau_nombre_clients = List.length liste_clients + 1
      and nombre_espaces = List.length liste_espaces
      in
      let tableau_client = Array.init nombre_espaces (fun _ -> creer_objet ())
      in
      (* mise  jour de la table *)
      table <-
         Array.concat
            [ Array.sub table 0 position_client ;
              [| tableau_client |] ;
              Array.sub 
                 table 
                 position_client (Array.length table - position_client) ] ;

      (* mise  jour de la liste des clients *)
      liste_clients <- 
         inserer_dans_liste_triee
            inferieur_sur_clients
            client liste_clients

   method ajouter_client = function client ->
      if not (List.mem client liste_clients) then
         self#ajouter_client_dans_table client ;
      if debug then (self#afficher_table ())

   method private supprimer_client_dans_table = function client ->
      let position_client = position_dans_liste client liste_clients
      in
      table <-
         Array.concat
            [ Array.sub table 0 position_client ;
              Array.sub table 
                 (position_client + 1) 
                 (Array.length table - position_client - 1) ] ;
      liste_clients <-
         supprimer_element_dans_liste client liste_clients

   method supprimer_client = function client ->
      if List.mem client liste_clients then
         self#supprimer_client_dans_table client ;
      if debug then (self#afficher_table ())

   (* ajout et suppression d'un espace ********************************)

   method private ajouter_espace_dans_table = function espace ->
      let ajouter_espace_client = function tableau_client ->
         let position_espace = position_dans_liste espace liste_espaces
         and tableau_espace = [| creer_objet () |]
         in
         Array.concat
            [ Array.sub tableau_client 0 position_espace ;
              tableau_espace ;
              Array.sub 
                 tableau_client 
                 position_espace (Array.length tableau_client - position_espace)]
      in
      for index_client = 0 to List.length liste_clients - 1 do
         table.(index_client) <- ajouter_espace_client table.(index_client)
      done ;

      liste_espaces <- 
         inserer_dans_liste_triee
            inferieur_sur_espaces
            espace liste_espaces

   method private supprimer_espace_dans_table = function espace ->
      let supprimer_espace_client = function tableau_client ->
         let position_espace =  position_dans_liste espace liste_espaces
         in
         Array.concat
            [  Array.sub tableau_client 0 position_espace ;
               Array.sub tableau_client 
                  (position_espace + 1)
                  (Array.length tableau_client - position_espace - 1) ]
      in
      for index_client = 0 to List.length liste_clients - 1 do
         table.(index_client) <- supprimer_espace_client table.(index_client)
      done ;

      liste_espaces <- 
         supprimer_element_dans_liste espace liste_espaces

   method ajouter_espace = function espace ->
      if not (List.mem espace liste_espaces) then
         self#ajouter_espace_dans_table espace ;
      if debug then (self#afficher_table ())


   method supprimer_espace = function espace ->
      if List.mem espace liste_espaces then
         self#supprimer_espace_dans_table espace ;
      if debug then (self#afficher_table ())


   (* affichage de la table *******************************************)

   method afficher_table = function () ->
      let rec chaine_n_espaces = function 
         | 0 -> ""
         | n -> " " ^ (chaine_n_espaces (n - 1))
      in
      let rec completer_nom = function nom -> function largeur ->
         if String.length nom >= largeur then
            String.sub nom 0 largeur
         else
            let manque = largeur - String.length nom
            in
            if manque mod 2 = 0 then
               (chaine_n_espaces (manque / 2))
               ^ nom ^ (chaine_n_espaces (manque / 2))
            else
               (chaine_n_espaces (manque / 2))
               ^ nom ^ (chaine_n_espaces (manque / 2 + 1))
      in
      let largeur_client = 10
      and largeur_espace = 9
      in
      let ligne = ref ""
      in
      (* affichage des informations prliminaires *)
      Printf.printf 
         "La liste des clients est de longueur %d, %d\n"
         (List.length liste_clients)
         (Array.length table) ;
      Printf.printf 
         "La liste des espaces est de longueur %d, %d\n"
         (List.length liste_espaces)
         (if Array.length table <> 0 then Array.length table.(0) else 0) ;
      
      (* affichage de la premire ligne : le noms des espaces *)
      ligne := (chaine_n_espaces largeur_client) ^ " |" ;
      for index = 0 to List.length liste_espaces - 1 do
         ligne := !ligne 
                  ^ (Printf.sprintf " %s |"
                        (completer_nom 
                            (nom_de_espace
                                (List.nth liste_espaces index))
                            largeur_espace))
      done ;
      print_endline !ligne ;

      (* affichage du reste du tableau *)
      for index_ligne = 0 to List.length liste_clients - 1 do
         (* affichage du nom du client *)
         ligne := (completer_nom 
                      (nom_de_client 
                          (List.nth liste_clients index_ligne))
                      largeur_client)
                  ^ " |";
         (* affichage des proprits du client *)
         for index_colonne = 0 to List.length liste_espaces - 1 do
            let valeur = 
               if est_present table.(index_ligne).(index_colonne) then
                  completer_nom "X" largeur_espace
               else
                  completer_nom "" largeur_espace
            in
            ligne := !ligne ^ (Printf.sprintf " %s |" valeur)
         done ;
         print_endline !ligne
      done
            

end 
;;

(*

(* les informations concernant l'identit d'un client
 * Identit (numero_client, nom_client) *)
type identite_client = Identite of int * string
;;


(* les espaces de discussion sont dtermins par leurs noms. *)
type espace_discussion = 
   | Espace_anonyme of string
   | Espace_non_anonyme of string
   | Espace_prive of string * (string list)
;;


let objet = 0 ;;
let action_sur_objet = function n -> function
   | true -> 1
   | false -> 0
;;

let est_present = function
   | 0 -> true
   | _ -> false
;;

let nom_de_client = function Identite (_, nom) -> nom
;;

let nom_de_espace = function
   | Espace_anonyme nom
   | Espace_non_anonyme nom
   | Espace_prive (nom, _) -> nom
;;

let inferieur_sur_clients = function c1 -> function c2 ->
   let c1 = nom_de_client c1
   and c2 = nom_de_client c2
   in
   if c1 < c2 then -1 else if c1 = c2 then 0 else 1
;;

let inferieur_sur_espaces = function e1 -> function e2 ->
   let e1 = nom_de_espace e1
   and e2 = nom_de_espace e2
   in
   if e1 < e2 then -1 else if e1 = e2 then 0 else 1
;;

let oc = new table_connexions 
            objet action_sur_objet 
            est_present 
            nom_de_client nom_de_espace 
            inferieur_sur_clients inferieur_sur_espaces ;;
 *)
