%{
   (*#####################################################################*
    *
    * Module parseur pour reprer les logins et les espaces
    *
    * Les rgles de production des valeurs des espaces
    *
    *#####################################################################*)


   (* la syntaxe abstraite nous est ncessaire pour construire une valeur
    * de type espace. *)
   open Types ;;

   (***************************************************************************
    * Exceptions
    ***************************************************************************)



   (***************************************************************************
    * Declaration de variables 
    ***************************************************************************)

   let messages = ref [] ;;

%}

/******************************************************************************/
/*************************** DEFINITION DES LEXEMES ***************************/
/******************************************************************************/

/* les lexmes transportant des valeurs */
%token <string> Chaine
%token <int> Entier

%token Anonyme NonAnonyme Prive Aucun Port
%token CrochetOuvrant CrochetFermant PointVirgule FinFichier Guillemet




/******************************************************************************/
/*********************** DEFINITION DES POINTS D'ENTREE ***********************/
/******************************************************************************/

/* les points d'entre de la grammaire */

%start analyser_espaces
%type <int * Types.espace_discussion list * ((Types.espace_discussion * (string option)) list)> analyser_espaces

%%
/* les fonctions accessibles du parseur */

   /* permet de construire une valeur de type Types.espaces list,  partir
    * d'une suite de lexmes. */
   analyser_espaces : PORT LISTE_ESPACES FinFichier { $1, $2, !messages } ;;


/******************************************************************************/
/************************* DEFINITION DE LA GRAMMAIRE *************************/
/******************************************************************************/

PORT : Port Entier                            { $2 }
;;

LISTE_ESPACES : 
        ESPACE                                { [$1] }
      | ESPACE LISTE_ESPACES                  { $1 :: $2 }
;;

ESPACE :
        Anonyme NOM_ESPACE MESSAGE            { let espace = Espace_anonyme $2 in messages := !messages @ [espace, $3] ; espace }
      | NonAnonyme NOM_ESPACE MESSAGE         { let espace = Espace_non_anonyme $2 in messages := !messages @ [espace, $3] ; espace }
      | Prive NOM_ESPACE LISTE_LOGINS MESSAGE { let espace = Espace_prive ($2, $3) in messages := !messages @ [espace, $4] ; espace }
;;

NOM_ESPACE : Chaine                           { Printf.printf "\nespace = \"%s\"\n" $1 ; flush stdout ; $1 }
;;

MESSAGE :
        Aucun                                 { Printf.printf "aucun message\n\n" ; None }
      | Chaine                                { Printf.printf "message = \"%s\"\n\n" $1 ; Some $1 }
;;

LISTE_LOGINS : CrochetOuvrant LISTE_PTVG CrochetFermant { $2 }
;;

LISTE_PTVG :                                  { [] }
      | LOGIN                                 { [ $1 ] }
      | LOGIN PointVirgule LISTE_PTVG         { $1 :: $3 }
;;

LOGIN : Chaine                                { $1 }

%%
